# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: pdp.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'pdp.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tpdp.proto\x12\x13policydecisionpoint\x1a\x1cgoogle/protobuf/struct.proto\"\'\n\x0bPolicyStore\x12\x0c\n\x04Kind\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\"\x9d\x01\n\tPrincipal\x12\x0c\n\x04Type\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\x12\x13\n\x06Source\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x1a\n\rIdentityToken\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x41\x63\x63\x65ssToken\x18\x05 \x01(\tH\x02\x88\x01\x01\x42\t\n\x07_SourceB\x10\n\x0e_IdentityTokenB\x0e\n\x0c_AccessToken\"B\n\x08\x45ntities\x12\x0e\n\x06Schema\x18\x01 \x01(\t\x12&\n\x05Items\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\"\x84\x01\n\x07Subject\x12\x0c\n\x04Type\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\x12\x13\n\x06Source\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x30\n\nProperties\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructH\x01\x88\x01\x01\x42\t\n\x07_SourceB\r\n\x0b_Properties\"e\n\x08Resource\x12\x0c\n\x04Type\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\x12\x30\n\nProperties\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x42\r\n\x0b_Properties\"W\n\x06\x41\x63tion\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\x30\n\nProperties\x18\x02 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x42\r\n\x0b_Properties\"\xeb\x01\n\x19\x41uthorizationModelRequest\x12\x0e\n\x06ZoneID\x18\x01 \x01(\x03\x12\x35\n\x0bPolicyStore\x18\x02 \x01(\x0b\x32 .policydecisionpoint.PolicyStore\x12\x36\n\tPrincipal\x18\x03 \x01(\x0b\x32\x1e.policydecisionpoint.PrincipalH\x00\x88\x01\x01\x12\x34\n\x08\x45ntities\x18\x04 \x01(\x0b\x32\x1d.policydecisionpoint.EntitiesH\x01\x88\x01\x01\x42\x0c\n\n_PrincipalB\x0b\n\t_Entities\"\xb4\x02\n\x11\x45valuationRequest\x12\x16\n\tRequestID\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x32\n\x07Subject\x18\x02 \x01(\x0b\x32\x1c.policydecisionpoint.SubjectH\x01\x88\x01\x01\x12\x34\n\x08Resource\x18\x03 \x01(\x0b\x32\x1d.policydecisionpoint.ResourceH\x02\x88\x01\x01\x12\x30\n\x06\x41\x63tion\x18\x04 \x01(\x0b\x32\x1b.policydecisionpoint.ActionH\x03\x88\x01\x01\x12-\n\x07\x43ontext\x18\x05 \x01(\x0b\x32\x17.google.protobuf.StructH\x04\x88\x01\x01\x42\x0c\n\n_RequestIDB\n\n\x08_SubjectB\x0b\n\t_ResourceB\t\n\x07_ActionB\n\n\x08_Context\"\xc5\x03\n\x19\x41uthorizationCheckRequest\x12J\n\x12\x41uthorizationModel\x18\x01 \x01(\x0b\x32..policydecisionpoint.AuthorizationModelRequest\x12\x16\n\tRequestID\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x32\n\x07Subject\x18\x03 \x01(\x0b\x32\x1c.policydecisionpoint.SubjectH\x01\x88\x01\x01\x12\x34\n\x08Resource\x18\x04 \x01(\x0b\x32\x1d.policydecisionpoint.ResourceH\x02\x88\x01\x01\x12\x30\n\x06\x41\x63tion\x18\x05 \x01(\x0b\x32\x1b.policydecisionpoint.ActionH\x03\x88\x01\x01\x12-\n\x07\x43ontext\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructH\x04\x88\x01\x01\x12;\n\x0b\x45valuations\x18\x07 \x03(\x0b\x32&.policydecisionpoint.EvaluationRequestB\x0c\n\n_RequestIDB\n\n\x08_SubjectB\x0b\n\t_ResourceB\t\n\x07_ActionB\n\n\x08_Context\"/\n\x0eReasonResponse\x12\x0c\n\x04\x43ode\x18\x01 \x01(\t\x12\x0f\n\x07Message\x18\x02 \x01(\t\"\x90\x01\n\x0f\x43ontextResponse\x12\n\n\x02ID\x18\x01 \x01(\t\x12\x38\n\x0bReasonAdmin\x18\x02 \x01(\x0b\x32#.policydecisionpoint.ReasonResponse\x12\x37\n\nReasonUser\x18\x03 \x01(\x0b\x32#.policydecisionpoint.ReasonResponse\"\x94\x01\n\x12\x45valuationResponse\x12\x10\n\x08\x44\x65\x63ision\x18\x01 \x01(\x08\x12\x16\n\tRequestID\x18\x02 \x01(\tH\x00\x88\x01\x01\x12:\n\x07\x43ontext\x18\x03 \x01(\x0b\x32$.policydecisionpoint.ContextResponseH\x01\x88\x01\x01\x42\x0c\n\n_RequestIDB\n\n\x08_Context\"\xda\x01\n\x1a\x41uthorizationCheckResponse\x12\x10\n\x08\x44\x65\x63ision\x18\x01 \x01(\x08\x12\x16\n\tRequestID\x18\x02 \x01(\tH\x00\x88\x01\x01\x12:\n\x07\x43ontext\x18\x03 \x01(\x0b\x32$.policydecisionpoint.ContextResponseH\x01\x88\x01\x01\x12<\n\x0b\x45valuations\x18\x04 \x03(\x0b\x32\'.policydecisionpoint.EvaluationResponseB\x0c\n\n_RequestIDB\n\n\x08_Context2\x87\x01\n\x0cV1PDPService\x12w\n\x12\x41uthorizationCheck\x12..policydecisionpoint.AuthorizationCheckRequest\x1a/.policydecisionpoint.AuthorizationCheckResponse\"\x00\x42:Z8github.com/permguard/permguard/internal/hosts/api/pdp/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pdp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z8github.com/permguard/permguard/internal/hosts/api/pdp/v1'
  _globals['_POLICYSTORE']._serialized_start=64
  _globals['_POLICYSTORE']._serialized_end=103
  _globals['_PRINCIPAL']._serialized_start=106
  _globals['_PRINCIPAL']._serialized_end=263
  _globals['_ENTITIES']._serialized_start=265
  _globals['_ENTITIES']._serialized_end=331
  _globals['_SUBJECT']._serialized_start=334
  _globals['_SUBJECT']._serialized_end=466
  _globals['_RESOURCE']._serialized_start=468
  _globals['_RESOURCE']._serialized_end=569
  _globals['_ACTION']._serialized_start=571
  _globals['_ACTION']._serialized_end=658
  _globals['_AUTHORIZATIONMODELREQUEST']._serialized_start=661
  _globals['_AUTHORIZATIONMODELREQUEST']._serialized_end=896
  _globals['_EVALUATIONREQUEST']._serialized_start=899
  _globals['_EVALUATIONREQUEST']._serialized_end=1207
  _globals['_AUTHORIZATIONCHECKREQUEST']._serialized_start=1210
  _globals['_AUTHORIZATIONCHECKREQUEST']._serialized_end=1663
  _globals['_REASONRESPONSE']._serialized_start=1665
  _globals['_REASONRESPONSE']._serialized_end=1712
  _globals['_CONTEXTRESPONSE']._serialized_start=1715
  _globals['_CONTEXTRESPONSE']._serialized_end=1859
  _globals['_EVALUATIONRESPONSE']._serialized_start=1862
  _globals['_EVALUATIONRESPONSE']._serialized_end=2010
  _globals['_AUTHORIZATIONCHECKRESPONSE']._serialized_start=2013
  _globals['_AUTHORIZATIONCHECKRESPONSE']._serialized_end=2231
  _globals['_V1PDPSERVICE']._serialized_start=2234
  _globals['_V1PDPSERVICE']._serialized_end=2369
# @@protoc_insertion_point(module_scope)
